/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamargs.h"
#include "gdamutil.h"
#include "gdamdefault.h"
#include <stdlib.h>

#define GDAM_BUTTON23_MASK (GdkModifierType)(GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)

/* Globals defined at startup */
int gdam_use_threads = 1;
char* gdam_dev_name = 0;
char* gdam_dev1_name = 0;
char* gdam_rcpath = 0;
int gdam_bufsize = 0;
int gdam_fragnum = 0;
int gdam_logfragsize = 0;

int gdam_num_options = 8;
const char* gdam_options[] = {
    "-nothreads",
    "-dev dev",
    "-dev1 dev",
    "-buflen buffer_length",
    "-drag2 / -drag3 / -drag23",
    "-songdb filename",
    "-rcfile filename",
    "-rcpath dirname",
    "-rclocations"
};

const char* gdam_option_descriptions[] = {
    "Don't use threads.",
    "Specify the output device.",
    "Specify the second output device.",
    "Set the audio device buffer to buffer_length.",
    "Use mouse button 2,3, or either to drag.",
    "Specify the songdata file.",
    "Use given resource file.",
    "Specify path for rc and xml files",
    "Print rcfile locations, then exit."
};

void gdam_print_usage(FILE* fp, const char* fmt)
{
    int i;
    for (i = 0; i < gdam_num_options; i++)
        fprintf(fp, fmt, gdam_options[i], gdam_option_descriptions[i]);
}

static const char* default_rcfiles[] = { ".gdamrc",
                                  "~/.gdam/gdamrc", "~/.gdamrc",
				  "/etc/gdam/gdamrc", "default.rc",
				  "~/.gdam/default.rc", 0 };

/*
 * This macro only works inside the main loop.
 * It deletes `n' arguments from the argc/argv
 * pair, remembering that they are actually pointers
 * to the real argc/v.  Also, we assume `i' is the
 * first argument to swallow.
 */
#define SWALLOW_ARG(n)  \
    memmove((*argv)+i, (*argv)+i+n, sizeof(char*)*((*argc)-i-(n)+1)); \
    (*argc) -= (n); i += 1; i-=n;\

/* Run this macro to assert that you have an argument to your flag. */
#define NEEDS_ARGUMENT(n) if (i + n >= *argc) {                              \
                 fprintf(stderr, "Needs %d %s after %s.\n",                  \
		          n, ((n==1)?"argument":"arguments"), (*argv)[i]);   \
		 return 0; }

gboolean gdam_parse_args(int* argc, char*** argv)
{
    int i;
    const char* rcfile = 0;
    fprintf(stderr, "GDAM client, alpha version\n");
    for (i = 1; i < *argc; i++) {
        if (strcmp((*argv)[i], "-nothreads") == 0) {
	    gdam_use_threads = 0;
	    SWALLOW_ARG(1);
	    continue;
	}
	if (strcmp((*argv)[i], "-rcfile") == 0) {
	    NEEDS_ARGUMENT(1);
	    rcfile = (*argv)[i+1];
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-rcpath") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_rcpath = (*argv)[i+1];
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-buflen") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_bufsize = (int)atof((*argv)[i+1]);
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-dev") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_dev_name = strdup((*argv)[i+1]);
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-dev1") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_dev1_name = strdup((*argv)[i+1]);
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-drag2") == 0) {
	    gdam_drag_mask = GDK_BUTTON2_MASK;
	    SWALLOW_ARG(1);
	    continue;
	}
	if (strcmp((*argv)[i], "-drag3") == 0) {
	    gdam_drag_mask = GDK_BUTTON3_MASK;
	    SWALLOW_ARG(1);
	    continue;
	}
	if (strcmp((*argv)[i], "-drag23") == 0) {
	    gdam_drag_mask = GDAM_BUTTON23_MASK;
	    SWALLOW_ARG(1);
	    continue;
	}
	if (strcmp((*argv)[i], "-fragnum") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_fragnum = (int)atof((*argv)[i+1]);
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-logfragsize") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_logfragsize = (int)atof((*argv)[i+1]);
	    SWALLOW_ARG(2);
	    continue;
	}
	if (strcmp((*argv)[i], "-songdb") == 0) {
	    NEEDS_ARGUMENT(1);
	    gdam_songdb_name = strdup((*argv)[i+1]);
	    SWALLOW_ARG(2);
	    continue;
	}
        if (strcmp((*argv)[i], "-rclocations") == 0) {
	    int i;
	    for (i = 0; default_rcfiles[i]; i++)
	        printf("%s\n", default_rcfiles[i]);
	    exit(0);
	}
	    
	if (strcmp((*argv)[i], "--") == 0)
	    break;
    }
    if (!rcfile && !(rcfile=gdam_default_file_find(default_rcfiles))) {
        fprintf(stderr, "Warning: no rcfile found.  Colors will be gray.\n");
    } else if (!gdam_file_exists(rcfile)){
        fprintf(stderr, "Warning: rcfile (%s) not found.\n", rcfile);
    } else {
	fprintf(stderr, "Using rcfile %s.\n", rcfile);
	gtk_rc_parse(rcfile);
    }

    return 1;
}
