/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamsonginfo.h"
#include <stdlib.h>

struct _GdamSongInfo {
    float          length;
    float          gain;
    int            order;
    GdamBeatInfo*  beat_info;
    GHashTable*    song_properties;
    char*          filename;
};

GdamSongInfo*        gdam_song_info_new         (const char*       filename,
                                                 float             bpm,
                                                 float             first_beat)
{
	GdamSongInfo* info = g_new0(GdamSongInfo, 1);
	info->filename = g_strdup(filename);
	info->beat_info = g_new0(GdamBeatInfo, 1);
	gdam_beat_info_set_bpm(info->beat_info, bpm);
	gdam_beat_info_set_first_beat(info->beat_info, first_beat);
	info->length = -1;
	info->gain = -1;
	info->order = -1;
	info->song_properties = g_hash_table_new(g_str_hash, g_str_equal);
	return info;
}

GdamSongInfo* gdam_song_info_copy (GdamSongInfo* song_info)
{
	GdamSongInfo* new_info;
	new_info = g_new(GdamSongInfo, 1);
	*new_info = *song_info;
	return new_info;
}

static gboolean parse_float(const char* txt, float* out)
{
	char* endp;
	*out = (float) strtod(txt, &endp);
	if (endp == txt || *endp)
		return FALSE;
	return TRUE;
}

void   	             gdam_song_info_set_length (GdamSongInfo*     song_info,
                                                gdouble           length)
{
	song_info->length = length;
}

gboolean             gdam_song_info_add_property(GdamSongInfo*     info,
                                                 const char*       key,
						 const char*       value)
{
	gpointer old;

	if (strcmp(key, "bpm") == 0) {
		float bpm;
		if (!parse_float(value, &bpm)) {
			g_warning("error parsing bpm `%s'", value);
			return FALSE;
		}
		gdam_beat_info_set_bpm(info->beat_info, bpm);
		return TRUE;
	} else if (strcmp(key, "firstbeat") == 0) {
		float first_beat;
		if (!parse_float(value, &first_beat)) {
			g_warning("error parsing first beat `%s'", value);
			return FALSE;
		}
		gdam_beat_info_set_first_beat(info->beat_info, 
					      first_beat);
		return TRUE;
	} else if (strcmp(key, "length") == 0) {
		if (!parse_float(value, &info->length)) {
			g_warning("error parsing length `%s'", value);
			return FALSE;
		}
		return TRUE;
	} else if (strcmp(key, "gain") == 0) {
		if (!parse_float(value, &info->gain)) {
			g_warning("error parsing gain `%s'", value);
			return FALSE;
		}
		return TRUE;
	}


	old = g_hash_table_lookup(info->song_properties, key);
	if (old) {
		g_hash_table_insert(info->song_properties,
		                    (gpointer) key,
				    g_strdup(value));
		g_free(old);
	} else {
		g_hash_table_insert(info->song_properties,
		                    g_strdup(key),
				    g_strdup(value));
	}
	return TRUE;
}


void gdam_song_info_set_bpm(GdamSongInfo* song_info, gdouble bpm)
{
	g_return_if_fail(song_info != NULL);
	g_return_if_fail(song_info->beat_info != NULL);
	gdam_beat_info_set_bpm(song_info->beat_info, bpm);
}
void gdam_song_info_set_first_beat(GdamSongInfo* song_info, gdouble first_beat)
{
	g_return_if_fail(song_info != NULL);
	g_return_if_fail(song_info->beat_info != NULL);
	gdam_beat_info_set_first_beat(song_info->beat_info, first_beat);
}


static void
kill_echo_song_property(gpointer key, gpointer value, gpointer unused)
{
	g_free(key); g_free(value);
}

const GHashTable*    gdam_song_info_get_properties
						(GdamSongInfo*     song_info)
{
	if (g_hash_table_size(song_info->song_properties))
		return song_info->song_properties;
	return NULL;
}

void gdam_song_info_destroy(GdamSongInfo* song_info)
{
	g_hash_table_foreach(song_info->song_properties,
	                     kill_echo_song_property,
			     NULL);
	g_hash_table_destroy(song_info->song_properties);
	g_free(song_info->filename);
	g_free(song_info);
}

GdamBeatInfo*   gdam_song_info_get_beat_info(GdamSongInfo*   song_info)
{
	return gdam_beat_info_copy(song_info->beat_info);
}

const char* 	             gdam_song_info_get_filename (GdamSongInfo*     song_info)
{
	return song_info->filename;
}

double 	             gdam_song_info_get_length (GdamSongInfo*     song_info)
{
	return song_info->length;
}

double 	             gdam_song_info_get_gain (GdamSongInfo*     song_info)
{
	return song_info->gain;
}

int 	             gdam_song_info_get_order (GdamSongInfo*     song_info)
{
	return song_info->order;
}

void 	 gdam_song_info_set_order   (GdamSongInfo* song_info, int rank)
{
	song_info->order = rank;
}
