/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamproducermp3cacheable.h"
#include "../gdamsongdatabase.h"
#include "../gdambuffermanager.h"
#include <gtk/gtksignal.h>
#include <math.h>


static void     gdam_producer_mp3_cacheable_destroy  (GtkObject*       object);

static GtkObjectClass* parent_class;

static void gdam_producer_mp3_cacheable_class_init(GdamProducerMp3CacheableClass* klass)
{
	GtkObjectClass* object_class = GTK_OBJECT_CLASS(klass);
	object_class->destroy = gdam_producer_mp3_cacheable_destroy;
	parent_class = gtk_type_class(GDAM_TYPE_PRODUCER_MP3);
}

static void gdam_producer_mp3_cacheable_init(GdamProducerMp3Cacheable* producer_mp3_cacheable)
{
}

GtkType gdam_producer_mp3_cacheable_get_type()
{
	static GtkType mp3_cacheable_type = 0;
	if (!mp3_cacheable_type) {
		static const GtkTypeInfo mp3_cacheable_info = {
			"GdamProducerMp3Cacheable",
			sizeof(GdamProducerMp3Cacheable),
			sizeof(GdamProducerMp3CacheableClass),
			(GtkClassInitFunc) gdam_producer_mp3_cacheable_class_init,
			(GtkObjectInitFunc) gdam_producer_mp3_cacheable_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		mp3_cacheable_type = gtk_type_unique(GDAM_TYPE_PRODUCER_MP3, 
						     &mp3_cacheable_info);
	}
	return mp3_cacheable_type;
}

GdamProducerMp3Cacheable* gdam_producer_mp3_cacheable_new(GdamChannel*	channel)
{
	GdamProducerMp3Cacheable* rv;
	rv = GDAM_PRODUCER_MP3_CACHEABLE(gtk_type_new(GDAM_TYPE_PRODUCER_MP3_CACHEABLE));
	((GdamModel*)rv)->channel = channel;
	return rv;
}

static void gdam_producer_mp3_cacheable_destroy(GtkObject* object)
{
	//GdamProducerMp3Cacheable* mp3_cacheable = GDAM_PRODUCER_MP3(object);
	//g_free(mp3_cacheable->filename);
	(*parent_class->destroy)(object);
}


void gdam_producer_mp3_cacheable_set_start(GdamProducerMp3Cacheable* mp3)
{
	mp3->cache_start = GDAM_PRODUCER(mp3)->stopped_position;
}

void gdam_producer_mp3_cacheable_set_beats(GdamProducerMp3Cacheable* mp3,
					   float 		     num_beats)
{
	mp3->cache_beats = num_beats;
}

static void buffer_create_success(int buffer_id, gpointer data)
{
	GdamBufferManagerInfo* info;
	info = gdam_buffer_manager_info_by_id(buffer_id);
	info->beat_info = (GdamBeatInfo*)data;
}

void gdam_producer_mp3_cacheable_export(GdamProducerMp3Cacheable* mp3)
{
	GdamChannel* channel;
	GdamModel* component;
	GdamRequest* request;
	int local_source_id;
	int num_args;
	GdamArg args[4];
	GdamRequestBuilder* request_builder;
        gdouble position = mp3->cache_start;
	gdouble length;
	GdamBeatInfo* beat_info = &GDAM_MODEL(mp3)->beat_info;
	GdamBeatInfo* new_beat_info;
	gboolean use_buffer_manager = TRUE;
	double beat_length;
	double beat_frac;
	double beat_number;
	double first_beat;
	char description[2048] = "";

	if (gdam_beat_info_get_bpm(beat_info, position) < 1)
		return;


	g_return_if_fail(mp3->cache_beats != 0);

	g_return_if_fail(mp3 != NULL);
	if (!GDAM_PRODUCER_MP3(mp3)->filename) {
		g_warning("gdam_producer_mp3_cache: no filename");
		return;
	}

	beat_length = gdam_beat_info_get_beat_length(beat_info, position);
	length = mp3->cache_beats * beat_length;

	component = GDAM_MODEL(mp3);
	channel = component->channel;
	g_return_if_fail(channel != NULL);

	args[0].type = GDAM_ARG_TYPE_STRING;
	args[0].name = "filename";
	GDAM_ARG_STRING(args + 0) = GDAM_PRODUCER_MP3(mp3)->filename;

	args[1].type = GDAM_ARG_TYPE_INT64;
	args[1].name = "position";
	GDAM_ARG_INT64(args + 1) = (gint64)(position * 44100.0);

	args[2].type = GDAM_ARG_TYPE_INT64;
	args[2].name = "length";
	GDAM_ARG_INT64(args + 2) = (gint64)(length * 44100.0);

	num_args = 3;
#if 0
	if (mp3->fade_envelope > 0) {
		args[num_args].type = GDAM_ARG_TYPE_INT;
		args[num_args].name = "fade_envelope";
		GDAM_ARG_INT(args + num_args) = mp3->fade_envelope;
		num_args++;
	}
#endif
	request = gdam_request_create_source_v(
				"GdamMp3",
				num_args,
				args,
				NULL,
				NULL,
				NULL);

	gdam_debug("Creating cached mp3 with position %f length %f", 
					position, length);
			
	request_builder = gdam_request_builder_new();
	local_source_id = gdam_request_builder_append_create(request_builder,
	                                                     request);

	first_beat = gdam_beat_info_get_first_beat(beat_info, position);
	beat_number = (position - first_beat) / beat_length;
	beat_frac = fmod(beat_number, beat_length);
	new_beat_info = gdam_beat_info_copy(beat_info);
	gdam_beat_info_set_first_beat(new_beat_info, 
				      (beat_length * (1.0 - beat_frac)));

	sprintf(description, "%3.2f  %2.2f  %s", position, length,
					GDAM_PRODUCER_MP3(mp3)->filename);

	gdam_buffer_manager_start_caching(request_builder,
					  local_source_id,
					  channel,
					  use_buffer_manager,
					  description,
					  buffer_create_success,
					  NULL,
					  new_beat_info);

}
