/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamseekablemp3.h"
#include "gdammp3.h"
#include <gdam/gdam-protocol.h>
#include <gdam/gdamarg.h>

static void gdam_seekable_mp3_destroy(GtkObject* seekable_mp3);
static gboolean    gdam_seekable_mp3_seek             (GdamSource*   source,
					               GdamTime      when,
					               int           whence);
static GdamTime    gdam_seekable_mp3_get_position     (GdamSource*   source);

static GtkObjectClass* parent_class = NULL;

static void gdam_seekable_mp3_set_filename(GtkObject* object, GdamArg* arg)
{
	GdamSeekableMp3* mp3 = GDAM_SEEKABLE_MP3(object);
	mp3->filename = g_strdup(GDAM_ARG_STRING(arg));
	mp3->stopped_length = -1;
}
static void gdam_seekable_mp3_get_filename(GtkObject* object, GdamArg* arg)
{
	GdamSeekableMp3* mp3 = GDAM_SEEKABLE_MP3(object);
	GDAM_ARG_STRING(arg) = g_strdup(mp3->filename);
}

static void gdam_seekable_mp3_set_length(GtkObject* object, GdamArg* arg)
{
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(object);
	GdamSeekableMp3* mp3 = GDAM_SEEKABLE_MP3(object);
	GdamTime length = GDAM_ARG_INT64(arg);
	mp3->stopped_length = length;
	if (proxy->cueing != NULL)
		GDAM_EXTERNAL(proxy->cueing)->samples_remaining = length;
	if (proxy->delegate != NULL)
		GDAM_EXTERNAL(proxy->delegate)->samples_remaining = length;
}
static void gdam_seekable_mp3_get_length(GtkObject* object, GdamArg* arg)
{
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(object);
	GdamSource* real_source = NULL;
	if (proxy->cueing)
		real_source = proxy->cueing;
	else if (proxy->delegate)
		real_source = proxy->delegate;
	else {
		GdamSeekableMp3* mp3 = GDAM_SEEKABLE_MP3(object);
		GDAM_ARG_INT64(arg) = mp3->stopped_length;
		return;
	}
	GDAM_ARG_INT64(arg) = GDAM_EXTERNAL(real_source)->samples_remaining;
}

static void gdam_seekable_mp3_set_buffer_size(GtkObject* object, GdamArg* arg)
{
	GdamProxySource* proxy_source = GDAM_PROXY_SOURCE(object);
	GdamSeekableMp3* seekable_mp3 = GDAM_SEEKABLE_MP3(object);
	int buffer_size = GDAM_ARG_INT(arg);
	if (buffer_size == 0)
		buffer_size = gdam_external_get_default_buffer_size();
	if (buffer_size <= 16) {
		gdam_debug("buffer size set too short");
		return;
	}
	if (proxy_source->cueing != NULL) {
		GdamExternal* external = GDAM_EXTERNAL(proxy_source->cueing);
		external->max_buffer_size = buffer_size;
	}
	if (proxy_source->delegate != NULL) {
		GdamExternal* external = GDAM_EXTERNAL(proxy_source->delegate);
		external->max_buffer_size = buffer_size;
	}
	seekable_mp3->buffer_size = buffer_size;
}
static void gdam_seekable_mp3_get_buffer_size(GtkObject* object, GdamArg* arg)
{
	GdamSeekableMp3* seekable_mp3 = GDAM_SEEKABLE_MP3(object);
	GDAM_ARG_INT(arg) = seekable_mp3->buffer_size;
}

static void gdam_seekable_mp3_done_constructing(GdamSource* source)
{
	GdamSeekableMp3* mp3 = GDAM_SEEKABLE_MP3(source);
	GdamMp3* new_source;
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(source);
	g_return_if_fail(proxy->delegate == NULL);
	gdam_debug("seekable_mp3: creating new mpg123 at %4.6gs",
			(gdouble)mp3->stopped_position / 44100.0);
	new_source = gdam_mp3_new(mp3->filename,
	                          mp3->stopped_position,
				  mp3->stopped_length);
	GDAM_EXTERNAL(new_source)->max_buffer_size = mp3->buffer_size;
	gdam_proxy_source_set(proxy, GDAM_SOURCE(new_source));
}


static void gdam_seekable_mp3_class_init(GtkObjectClass* object_class)
{
	GdamSourceClass* source_class = (GdamSourceClass*)object_class;
	object_class->destroy = gdam_seekable_mp3_destroy;
	source_class->seek = gdam_seekable_mp3_seek;
	source_class->done_constructing = gdam_seekable_mp3_done_constructing;
	source_class->get_position = gdam_seekable_mp3_get_position;

	gdam_arg_register(object_class,
	                  "filename",
			  GDAM_ARG_TYPE_STRING,
			  gdam_seekable_mp3_set_filename,
			  gdam_seekable_mp3_get_filename,
			  GDAM_ARG_VITAL);
	gdam_arg_register(object_class,
	                  "length",
			  GDAM_ARG_TYPE_INT64,
			  gdam_seekable_mp3_set_length,
			  gdam_seekable_mp3_get_length,
			  GDAM_ARG_VITAL);
	gdam_arg_register(object_class,
	                  "buffer_size",
			  GDAM_ARG_TYPE_INT,
			  gdam_seekable_mp3_set_buffer_size,
			  gdam_seekable_mp3_get_buffer_size,
			  GDAM_ARG_VITAL);
}

static void gdam_seekable_mp3_init(GdamSeekableMp3* mp3)
{
	mp3->stopped_length = -1;
	mp3->buffer_size = gdam_external_get_default_buffer_size();
}
GtkType gdam_seekable_mp3_get_type()
{
	static GtkType source_type = 0;
	if (!source_type) {
		static const GtkTypeInfo source_info = {
			"GdamSeekableMp3",
			sizeof(GdamSeekableMp3),
			sizeof(GdamProxySourceClass),
			(GtkClassInitFunc) gdam_seekable_mp3_class_init,
			(GtkObjectInitFunc) gdam_seekable_mp3_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent_type = GDAM_TYPE_PROXY_SOURCE;
		parent_class = gtk_type_class(parent_type);
		source_type = gtk_type_unique(parent_type, &source_info);
	}
	return source_type;
}

static void gdam_seekable_mp3_destroy(GtkObject* object)
{
	GdamSeekableMp3* mp3 = GDAM_SEEKABLE_MP3(object);
	g_free(mp3->filename);
	mp3->filename = NULL;
	parent_class->destroy(object);
}

void             gdam_seekable_mp3_seek_absolute(GdamSeekableMp3* mp3,
                                                 GdamTime      samples)
{
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(mp3);
	GdamSource* delegate = proxy->delegate;
	GdamMp3* new_source;
	if (!delegate) {
		mp3->stopped_position = samples;
		return;
	}
	new_source = gdam_mp3_new(mp3->filename, samples, -1);
	GDAM_EXTERNAL(new_source)->max_buffer_size = mp3->buffer_size;
	gdam_proxy_source_set_cue(proxy, GDAM_SOURCE(new_source));
}

void             gdam_seekable_mp3_seek_relative(GdamSeekableMp3* mp3,
                                                 GdamTime      samples)
{
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(mp3);
	GdamTime at;
	GdamSource* delegate = proxy->delegate;
	GdamMp3* new_source;
	if (!delegate) {
		mp3->stopped_position += samples;
		return;
	}
	at = gdam_source_get_position(proxy->delegate);
	new_source = gdam_mp3_new(mp3->filename, at + samples, -1);
	GDAM_EXTERNAL(new_source)->max_buffer_size = mp3->buffer_size;
	gdam_proxy_source_set_cue(proxy, GDAM_SOURCE(new_source));
}

static GdamTime gdam_seekable_mp3_get_position(GdamSource* source)
{
	GdamSeekableMp3* seekable_mp3 = GDAM_SEEKABLE_MP3(source);
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(source);
	if (proxy->delegate == NULL)
		return seekable_mp3->stopped_position;
	return gdam_source_get_position(proxy->delegate);
}

static gboolean gdam_seekable_mp3_seek(GdamSource*   source,
				      GdamTime      when,
				      int           whence)
{
	GdamSeekableMp3* seekable_mp3 = GDAM_SEEKABLE_MP3(source);
	GdamProxySource* proxy = GDAM_PROXY_SOURCE(source);
	if (proxy->delegate == NULL) {
		switch (whence) {
			case 0: seekable_mp3->stopped_position = when;
			        return TRUE;
			case 1: seekable_mp3->stopped_position += when;
				gdam_debug("seekable_mp3... nudging before "
						"starting %.4gs",
						(gdouble)when/44100.0);
			        return TRUE;
		}
	}
	if (whence == 1) {
		gdam_seekable_mp3_seek_relative(GDAM_SEEKABLE_MP3(source),
		                                when);
		return TRUE;
	}
	if (whence == 0) {
		gdam_seekable_mp3_seek_absolute(GDAM_SEEKABLE_MP3(source),
		                                when);
		return TRUE;
	}
	gdam_debug("couldn't seek mp3 from end-of-file");
	return FALSE;
}
